window.addEventListener('DOMContentLoaded', async event => {
    // window.console.assert(window.document.readyState !== 'loading');
    // window.console.info("'window' object properties: " + Object.getOwnPropertyNames(globalThis).join("-"));
    // ES module technology only:
    // window.console.info("JavaScript executable file: " + import.meta.url);
    //
    // window.fetch("./build/Barcode.wasm").then(bytes => bytes.arrayBuffer()).then(async buffer => {
    //     window.console.assert(buffer instanceof ArrayBuffer);
    //     const module = await WebAssembly.compile(buffer);
    //     window.console.assert(module instanceof WebAssembly.Module);
    //     // Etc.
    // });

    const module = await WebAssembly.compileStreaming(window.fetch("./build/Barcode.wasm"));
    window.console.assert(module instanceof window.WebAssembly.Module);
    const _imports = WebAssembly.Module.imports(module); // Array of declared imports...
    // for (const _import of _imports)
    //     window.console.log(Object.getOwnPropertyNames(_import).join("-"));
    /** Compilation is such that 'abort' is bypassed:
     * 'asc assembly/Barcode.ts --target release --use abort=assembly/Barcode/_Abort'
     * As a result, AssemblyScript compiler no longer imports 'abort'...
     * (see also https://www.assemblyscript.org/concepts.html#special-imports)
     */
    // Caution, be sure to evict any import:
    window.console.assert(_imports.length === 0);

    WebAssembly.instantiate(module).then(executable => {
        window.console.assert(executable instanceof WebAssembly.Instance);
        // Memory provided by WebAssembly module:
        const memory = executable.exports.memory;
        window.console.assert(memory instanceof WebAssembly.Memory);
        const buffer = memory.buffer;
        window.console.assert(buffer instanceof ArrayBuffer);
        // Alter WebAssembly module memory in JavaScript:
        const barcode = new Uint16Array(buffer); // 'ArrayBuffer' objects cannot be directly handled...
        window.console.assert(typeof executable.exports.__new === 'function');
        const pointer = executable.exports.__new("4719512002889".length << 1, 2) >>> 0;
        for (let i = 0; i < "4719512002889".length; i++)
            barcode[(pointer >>> 1) + i] = "4719512002889".charCodeAt(i);
        window.console.assert("Barcode_checksum" in executable.exports);
        window.alert(executable.exports.Barcode_checksum(pointer)); // Display is '0' meaning "OK"...
    }, error => window.console.error(error));

    // '_import' is *REQUIRED* in 'WebAssembly.instantiate' only if the instantiated module actually imports something.
// https://stackoverflow.com/questions/48039547/webassembly-typeerror-webassembly-instantiation-imports-argument-must-be-pres

// (import "a" "f" (func ...))
// (import "a" "g" (func ...))
// (import "b" "m" (memory 0))
// {
//     a: {f: function (...) { ... }, g: function (...) {...}},
//     b: {m: new WebAssembly.Memory(...)}
// }
    // const _import = {env: {abort: (message, fileName, lineNumber, columnNumber) => {window.console.error(message); /* Etc. */}}};
    // WebAssembly.instantiate(module, _import).then(executable => { // Etc.
});






