import {readFile} from 'node:fs/promises';
import {env} from 'node:process';
import {WASI} from 'wasi';

process.removeAllListeners('warning'); // '(node:18916) ExperimentalWarning: WASI is an experimental feature. This feature could change at any time'
console.info("\t(Node.js host) Current directory: " + process.cwd());
console.info("\t(Node.js host) Barcode: " + process.argv[2]);

const wasi = new WASI({
    args: process.argv,
    env, // Caution here: 'process.env' is *ACTUALLY* passed to WASI...
    preopens: {
        '/sandbox': process.cwd() // Access to resource(s) in current directory...
    },
    returnOnExit: true // 'wasi.start()' returns the exit code to Node.js...
});
const wasm = await WebAssembly.compile(await readFile(new URL('./build/Barcode.wasm', import.meta.url)));
// Imported object is required, i.e., the API itself to access resource(s):
const instance = await WebAssembly.instantiate(wasm, {wasi_snapshot_preview1: wasi.wasiImport});
// Node.js WASI API imposes that the start method must be explicitly named '_start':
const checksum = wasi.start(instance); // '0', '-1' or '-2'
console.info("\t(Node.js host) Checksum: " + checksum);
process.exit(checksum); // Node.js returns the exit code to Windows ('Write-Output $?' -> 'True' or 'False')...

